/******************************************************
 *             		 CTXTDlg.cpp
 *
 * Project : MergeKW                       Ver : 1.0
 * Programmer : Michael Taylor	     Date : 8/16/98
 * Language : VC 5.0		            Target : Win32
 * Purpose : This file implements the TXT dialog.  It
 *           provides the necessary routines to read and
 *           modify the application INI file, display
 *           the contents of the INI file, and merge
 *           the selected files into a new USERTYP.DAT
 *           file.
 *****************************************************/
//@DOC

#include "stdafx.h"
#include <afxdlgs.h>
#include <atlbase.h>
#include "MergeKW.h"
#include "CFileData.h"
#include "CTXTDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern TCHAR g_lpszLastDir[MAX_PATH];

/*@FUNC
 *   This function handles support for an initialization
 * file that may be used to add some initial entries
 * to the registry for the add-in.
 *@COMM
 *   This function firsts attempts to locate the init
 * file.  If it is found then it is read and each specified
 * file is added to the 'selected' key in the registry.  
 *@RDESC
 *   None.
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   8/27/98  Original version
 */   
void HandleInitFile ( void )
{
   CString szFile, szLine, szDescription;
   int iPos;
   CStdioFile File;   
   CFileData* pData;
   CObArray FileList;

   //Build up the init file name
   szFile = BuildPath(GetAddInDir(), IDS_INITFILE);  
      
   //Attempt to open the file
   if (File.Open(szFile, CFile::modeNoTruncate | CFile::typeText |
                 CFile::shareDenyWrite | CFile::modeRead))
   {     
      //Read each line  
      while (File.ReadString(szLine))
      {
         szLine.TrimLeft();
         if (szLine.IsEmpty())
            continue;

         //Create the instance
         pData = new CFileData;  
         pData->SetSelected();

         //Parse the line
         iPos = szLine.Find(TEXT('='));
         if (iPos != -1)
         {
            //Get the file
            szFile = szLine.Left(iPos);
            szFile.TrimLeft();
            szFile.TrimRight();
         
            //Attach the path to the addin
            szFile = BuildPath(GetAddInDir(), szFile);
            pData->SetFileName(szFile);

            //Get the description
            szDescription = szLine.Mid(iPos + 1);
            szDescription.TrimLeft();
            szDescription.TrimRight();
            pData->SetDescript(szDescription);

            //Add it to the list
            FileList.Add(pData);            
         };
      };
   
      //Write everything to the registry
      WriteFileList(FileList);

      //Clean up      
      File.Close();
      
      CFileData* pData;
      DWORD dwSize = FileList.GetSize();
      for (DWORD dwIdx = 0;
           dwIdx < dwSize;
           dwIdx++)
      {
         pData = MAKE_FILEDATA(FileList[dwIdx]);
         delete pData;
      };
      FileList.RemoveAll();
   };   
};

/////////////////////////////////////////////////////////////////////////////
// CTXTDlg dialog

CTXTDlg::CTXTDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CTXTDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CTXTDlg)
   m_bAutoMerge = FALSE;
	//}}AFX_DATA_INIT
}

CTXTDlg::~CTXTDlg ( )
{
   CFileData* pData  = NULL;

   //Clean up
   DWORD dwSize = m_FileDataList.GetSize();
   for (DWORD dwIdx = 0;
        dwIdx < dwSize;
        dwIdx++)
   {
      pData = MAKE_FILEDATA(m_FileDataList[dwIdx]);
      if (pData)
         delete pData;      
   };
   m_FileDataList.RemoveAll();
};

void CTXTDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTXTDlg)
	DDX_Control(pDX, ID_LV_TXTLIST, m_FileList);
   DDX_Check(pDX, ID_CB_AUTOMERGE, m_bAutoMerge);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CTXTDlg, CDialog)
	//{{AFX_MSG_MAP(CTXTDlg)
	ON_BN_CLICKED(ID_ADD, OnAdd)
	ON_BN_CLICKED(ID_REMOVE, OnRemove)
	ON_BN_CLICKED(ID_MERGE, OnMerge)
   ON_BN_CLICKED(IDOK, OnOK)
   ON_BN_CLICKED(IDCLOSE, OnCancel)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTXTDlg message handlers

/*@MFUNC
 *   Initializes the dialog
 *@COMM
 *   This method initializes the dialog.  It fills in the
 * list view as well by reading the list of files and 
 * descriptions from the registry.
 *@RDESC
 *   Returns TRUE to continue initializing the dialog,
 * or FALSE otherwise.
 *@XREF
 *   <c CTXTDlg>
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   8/17/98  Original version
 *   1/30/99  Modified registry entries
 */   
BOOL CTXTDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

   //Read the file list
   if (!ReadFileList(m_FileDataList))
      return(false);

   //Read the auto-merge flag
   CRegKey Key;
   m_bAutoMerge = false;
   if (Key.Open(HKEY_CURRENT_USER, BASE_KEY) == ERROR_SUCCESS)
   {  
      DWORD dwValue = 0;
      Key.QueryValue(dwValue, AUTOMERGE_VALUE);
      if (dwValue != 0)
         m_bAutoMerge = TRUE;
      Key.Close();
   };
   UpdateData(FALSE);

   //Update the display
   InitList();
   return(true);
};

/*@MFUNC
 *   Fills the list box with the current contents
 * of the file data list.
 *@RDESC
 *   None.
 *@XREF
 *   <c CTXTDlg>
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   1/31/99  Original version
 */   
void CTXTDlg::InitList ( void )
{
   CString szTitle;
   int iItem;
   CFileData* pData = NULL;

   //Wipe out whatever is already there
   m_FileList.ResetContent();

   //Prep the list
   DWORD dwSize = m_FileDataList.GetSize();
   m_FileList.InitStorage(dwSize, (dwSize * 20));

   //For each file
   for (DWORD dwIdx = 0;
        dwIdx < dwSize;
        dwIdx++)
   {
      pData = MAKE_FILEDATA(m_FileDataList[dwIdx]);
      if (!pData)
         continue;

      //Build up a nice name
      szTitle.Format(IDS_LISTENTRY, pData->GetDescription(),
                     pData->GetFileName());
      iItem = m_FileList.AddString(szTitle); 
      m_FileList.SetItemData(iItem, dwIdx);      

      //If it is selected, then check the box
      if (pData->IsSelected())
         m_FileList.SetCheck(iItem, 1);
   };
};

/*@MFUNC
 *   Handles the 'Add' button in the dialog.
 *@COMM
 *   This method prompts the user for a new file to
 * add to the dialog.  It adds the file and description
 * to the dialog.  Note that it does not update the
 * registry as this is handled when the dialog is closed.
 *@RDESC
 *   None.
 *@XREF
 *   <c CTXTDlg>  <mf CTXTDlg::OnRemove>
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   8/20/98  Original version
 */   
void CTXTDlg::OnAdd() 
{
   CString szDefExt, szExtFilter;
   DWORD dwFlags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | 
                   OFN_NONETWORKBUTTON | OFN_PATHMUSTEXIST |
                   OFN_CREATEPROMPT;

   //Update the current list
   UpdateListData();

   //Display the standard File Open dialog with TXT files
   //displayed
   szDefExt.LoadString(IDS_DEFAULTEXT);
   szExtFilter.LoadString(IDS_EXTFILTER);
   CFileDialog FileDlg(TRUE, szDefExt, NULL, dwFlags, szExtFilter, this);
   FileDlg.m_ofn.lpstrInitialDir = g_lpszLastDir;
   if (FileDlg.DoModal() != IDOK)
      return;

   //Prompt for a description
   CDescriptDlg DescriptDlg;
   if (DescriptDlg.DoModal() != IDOK)
      return;

   //Create a new instance
   CFileData* pData = new CFileData;
   if (!pData)
   {
      DisplayMsg(ERROR_MSG, IDS_OUTOFMEMORY);
      return;
   };
   pData->SetFileName(FileDlg.GetPathName());
   pData->SetDescript(DescriptDlg.GetDescription());

   //Call the worker function
   Add(pData);

   //Flush the directory changes
   CString szDir = FileDlg.GetPathName();
   CString szFile = FileDlg.GetFileName();
   int iPos = szDir.Find(szFile);
   if (iPos != -1)
      szDir = szDir.Left(iPos - 1);
   lstrcpy(g_lpszLastDir, szDir);                           
};

/*@MFUNC
 *   Adds a new entry to the list and updates the display.
 *@RDESC
 *   None.
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   8/21/98  Original version 
 *   1/31/99  Modified to use CFileData
 */
void CTXTDlg::Add ( 
               CFileData* pData        //@PARM [IN] Data
                  )
{
   WIN32_FIND_DATA FindData;
   HANDLE hFind = INVALID_HANDLE_VALUE;
   CString szMsg, szFile;

   if (!pData)
      return;
         
   //If the file name is empty, no reason to continue
   szFile = pData->GetFileName();
   szFile.TrimLeft();
   szFile.TrimRight();
   if (szFile.IsEmpty())
      return;

   //Make sure the file exists and it isn't a directory
   hFind = FindFirstFile(szFile, &FindData);
   if (hFind != INVALID_HANDLE_VALUE)
   {
      //Make sure it isn't a directory
      if ((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY)
      {                
         DisplayMsg(ERROR_MSG, IDS_FILENOTEXIST);
         return;      
      };
      FindClose(hFind);
   };
   
   //Ensure the entry doesn't already exist
   CFileData* pEnumData = NULL;
   DWORD dwSize = m_FileDataList.GetSize();
   for (DWORD dwIdx = 0;
        dwIdx < dwSize;
        dwIdx++)
   {
      pEnumData = MAKE_FILEDATA(m_FileDataList[dwIdx]);
      if (pEnumData && (pEnumData->GetFileName().CompareNoCase(szFile) == 0))
      {
         DisplayMsg(ERROR_MSG, IDS_FILEALREADYEXISTS);
         return;                  
      };
   };
 
   //Add it
   m_FileDataList.Add(pData);

   //Update the display
   InitList();
};

/*@MFUNC
 *   Handles the 'Remove' button in the dialog.
 *@COMM
 *   This method removes the given file from the list.
 *@RDESC
 *   None.
 *@XREF
 *   <c CTXTDlg>  <mf CTXTDlg::OnAdd>
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   8/20/98  Original version
 */   
void CTXTDlg::OnRemove() 
{
   //Update the current list
   UpdateListData();

   //Get the selected item's data
   int iItem = m_FileList.GetCurSel();
   int iIndex = m_FileList.GetItemData(iItem);
   if (iIndex != -1)
   {
      //Remove it from the master list
      CFileData* pData = MAKE_FILEDATA(m_FileDataList.GetAt(iIndex)); 
      if (pData)
         delete pData;
      m_FileDataList.RemoveAt(iIndex);
   };
         
   //Update the list
   InitList();   
}

/*@MFUNC
 *   Handles the 'Merge' button in the dialog.
 *@COMM
 *   This method merges the selected files into a new
 * USERTYPE.DAT file.  
 *@RDESC
 *   None.
 *@XREF
 *   <c CTXTDlg> 
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   8/20/98  Original version
 */   
void CTXTDlg::OnMerge() 
{
   CString szFile;
   CStdioFile File;
   CFileData* pData = NULL;
   CString szMSDevDir = GetMSDevDir();
   CWaitCursor WaitCursor;

   //Update the file data based on the current settings in the list
   UpdateListData();

   //Merge the selected files
   MergeFiles();
};

/*@MFUNC
 *   Sets the selection state of the file data based
 * on the current state of the corresponding list item.
 *@RDESC
 *   None.
 *@XREF
 *   <c CTXTDlg>
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   1/31/99  Original version
 */   
void CTXTDlg::UpdateListData ( void )
{
   DWORD dwIdx;

   //Walk the list	
   CFileData* pData = NULL;
   DWORD dwSize = m_FileList.GetCount();
   for (DWORD dwItem = 0;
        dwItem < dwSize;
        dwItem++)
   {
      //Get the data
      dwIdx = m_FileList.GetItemData(dwItem);
      pData = MAKE_FILEDATA(m_FileDataList[dwIdx]);
      if (!pData)
         continue;

      //Update the state of the file data based on
      //the state of the check box
      pData->SetSelected(m_FileList.GetCheck(dwItem) == 1);
   };
};

/*@MFUNC
 *   Handles the DONE button.
 *@COMM
 *   This method saves the contents of the list and
 * closes the dialog.
 *@RDESC
 *   None.
 *@XREF
 *   <c CTXTDlg>
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   1/31/99  Original version
 */   
void CTXTDlg::OnOK ( void )
{  
   UpdateData();

   CFileData* pData = NULL;

   //Delete the old registry entries
   CleanReg();
  
   //Update the file data with the current list box settings
   UpdateListData();

   //Update the registry
   WriteFileList(m_FileDataList);
   
   //Write the auto-merge flag
   CRegKey Key;
   if (Key.Open(HKEY_CURRENT_USER, BASE_KEY) == ERROR_SUCCESS)
   {
      Key.SetValue((m_bAutoMerge == TRUE) ? 1 : 0, AUTOMERGE_VALUE);
      Key.Close();
   };

   //Done
   CDialog::OnOK();
};

/*@MFUNC
 *   Closes the dialog without saving.
 *@RDESC
 *   None.
 *@XREF
 *   <c CTXTDlg>
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   1/31/99  Original version
 */   
void CTXTDlg::OnCancel ( void )
{
   EndDialog(-1);
};

/////////////////////////////////////////////////////////////////////////////
// CDescriptDlg dialog

CDescriptDlg::CDescriptDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CDescriptDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDescriptDlg)
	m_szDescription = _T("");
	//}}AFX_DATA_INIT
}


void CDescriptDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDescriptDlg)
	DDX_Text(pDX, ID_TF_DESCRIPTION, m_szDescription);
	DDV_MaxChars(pDX, m_szDescription, 100);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CDescriptDlg, CDialog)
	//{{AFX_MSG_MAP(CDescriptDlg)
		// NOTE: the ClassWizard will add message map macros here
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDescriptDlg message handlers