/******************************************************
 *             		 Commands.cpp
 *
 * Project : MergeKW                       Ver : 1.0
 * Programmer : Michael Taylor	     Date : 8/16/98
 * Language : VC 5.0		            Target : Win32
 * Purpose : This file implements the IDispatch interface
 *           for the add-in.  It implements the 
 *           commands supported by this add-in.
 ****************************************************/
//@DOC

#include "stdafx.h"
#include <afxdlgs.h>
#include "MergeKW.h"
#include "CFileData.h"
#include "Commands.h"
#include "CTXTDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//Forward reference
void AppendToFile ( CString szFileName, CString szText );

//Cache for the last directory used to save the keywords
TCHAR g_lpszLastDir[MAX_PATH] = TEXT("");

CCommands::CCommands()
{
	m_pApplication = NULL;

   //MODIFIED : MLT - 7/3/99 Get the last directory from the registry
   CRegKey DirKey;
   if (DirKey.Open(HKEY_CURRENT_USER, BASE_KEY) == ERROR_SUCCESS)
   {
      DWORD dwSize = sizeof(g_lpszLastDir);
      DirKey.QueryValue(g_lpszLastDir, PATH_VALUE, &dwSize);
      DirKey.Close();      
   };
}

CCommands::~CCommands()
{
   //MODIFIED : MLT - 7/3/99 Save the last directory to the 
   //registry
   CRegKey DirKey;
   if (DirKey.Open(HKEY_CURRENT_USER, BASE_KEY) == ERROR_SUCCESS)
   {
      DirKey.SetValue(g_lpszLastDir, PATH_VALUE);

      //MODIFIED : MLT - 7/3/99 If auto-merge is set
      DWORD dwValue = 0;
      if (DirKey.QueryValue(dwValue, AUTOMERGE_VALUE) == ERROR_SUCCESS)
      {
         if (dwValue != 0)
            MergeFiles(false);   
      };   
      
      DirKey.Close();
   };

	ASSERT (m_pApplication != NULL);
	m_pApplication->Release();
}

void CCommands::SetApplicationObject(IApplication* pApplication)
{
	// This function assumes pApplication has already been AddRef'd
	//  for us, which CDSAddIn did in its QueryInterface call
	//  just before it called us.
	m_pApplication = pApplication;
}


/*@FUNC
 *   This function handles the add-ins command.
 *@COMM
 *   The function prompts the user for the list
 * of TXT files to merge into the USERTYPE.DAT file.
 *@PAR
 *   In order to remember what files have already
 * been merged, and their descriptions; an INI file
 * is used.  This simplifies the design.  A user
 * can add and remove TXT files from the list via the
 * dialog box.  When the user selects 'Merge', the TXT
 * files are copied verbatim into the USERTYPE.DAT file.
 *  The old file is deleted.  The user must restart the
 * IDE to get the changes to take effect.
 *@RDESC
 *   Results
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   8/16/98  Original version
 */   
STDMETHODIMP CCommands::MergeKWCommandMethod() 
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_FALSE));

   //Display the dialog containing the TXT files.
   CTXTDlg TXTDlg;

   TXTDlg.DoModal();

   //Done
	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
	return S_OK;
}

/*@FUNC
 *   This command provides the user with a quick way to
 * add a new item to one of the TXT files.  It takes the
 * currently selected text from the text editor and asks
 * what file to place the item into.
 *@RDESC
 *   None.
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   8/31/98  Original version
 */   
STDMETHODIMP CCommands::AddKWCommandMethod() 
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_FALSE));

   //Get the active document
   CComPtr<IDispatch> pActiveDoc = NULL;
   m_pApplication->get_ActiveDocument(&pActiveDoc);
   if (pActiveDoc)
   {      
      CComQIPtr<ITextDocument,&IID_ITextDocument> pTextDoc(pActiveDoc); 
      if (pTextDoc)
      {
         //Get the selected text
         CComPtr<IDispatch> pSelection = NULL;
         pTextDoc->get_Selection(&pSelection);
         if (pSelection)
         {
            CComQIPtr<ITextSelection,&IID_ITextSelection> pTextSel(pSelection);
            if (pTextSel)
            {
               //If there is any text
               CComBSTR BStrSel;
               pTextSel->get_Text(&BStrSel);

               //Get the text and clean it up
               USES_CONVERSION;   
               CString Sel = OLE2T(BStrSel);
               Sel.TrimLeft();
               Sel.TrimRight();
               
               //If the selected text is empty, then done
               if (!Sel.IsEmpty())
               {   
                  //Display a browse dialog so the user can pick the TXT file
                  //to add it to
                  CString szExt, szFilter, szTitle;
                  DWORD dwFlags = OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST |
                                  OFN_HIDEREADONLY | OFN_NOREADONLYRETURN |
                                  OFN_CREATEPROMPT;

                  szExt.LoadString(IDS_DEFAULTEXT);
                  szFilter.LoadString(IDS_EXTFILTER);
                  CFileDialog FileDlg(TRUE, szExt, NULL, dwFlags, szFilter, NULL);
                  szTitle.Format(IDS_FMTFILE_SELECT, Sel);
                  FileDlg.m_ofn.lpstrTitle = szTitle;
                  FileDlg.m_ofn.lpstrInitialDir = g_lpszLastDir;
                  if (FileDlg.DoModal() == IDOK)
                  {
                     CString szDir = FileDlg.GetPathName();
                     AppendToFile(szDir, Sel);

                     //Save the directory
                     CString szFile = FileDlg.GetFileName();
                     int iPos = szDir.Find(szFile);
                     if (iPos != -1)
                        szDir = szDir.Left(iPos - 1);
                     lstrcpy(g_lpszLastDir, szDir);                           
                  };
               };    //Sel !Empty
            };    // pTextSel != NULL
         };    // pSelection != NULL
      };    // pTextDoc != NULL
   };    // pActiveDoc != NULL

   //Done
	VERIFY_OK(m_pApplication->EnableModeless(VARIANT_TRUE));
	return(S_OK);
}

/*@FUNC
 *   Appends the specified text to the end of the given file.
 *@RDESC
 *   None.
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   8/31/98  Original version
 */   
void AppendToFile ( const CString szFileName, const CString szText )
{
   //Open the file
   CStdioFile File;
   
   if (!File.Open(szFileName, CFile::modeWrite | CFile::shareExclusive |
                  CFile::typeText | CFile::modeCreate | CFile::modeNoTruncate))
   {
      DisplayMsg(ERROR_MSG, IDS_FMTOPEN_FAILED, szFileName);
      return;
   };
   File.SeekToEnd();

   //Write out the text
   CString szLine = TEXT("\n") + szText;
   File.WriteString(szLine);

   //Clean up
   File.Close();
};

/*@FUNC
 *   Merges all the currently selected files into
 * a new file.
 *@COMM
 *   This method is called whenever the user selects
 * to merge the files or if "auto-merge" is enabled and
 * MSDev is shut down. 
 *@PAR
 *   If <p bGUI> is TRUE, the default, then errors generate
 * message boxes, otherwise no messages are displayed.
 *@RDESC
 *   Returns TRUE if successful, or FALSE otherwise.
 *@AUTHOR
 *   Michael Taylor 
 *@HISTORY
 *   7/3/99  Original version
 */   
bool MergeFiles (
                bool bGUI     //@PARM [IN] Enable GUI
                )
{
   //Create a new temporary DAT file based on where MSDevDir is
   CStdioFile File;
   CString szMSDevDir = GetMSDevDir();
   CString szFile = BuildPath (szMSDevDir, IDS_TEMPFILE);
   if (!File.Open(szFile, CFile::modeCreate | CFile::shareDenyWrite |
                  CFile::modeWrite | CFile::typeText))
   {
      if (bGUI)
         DisplayMsg(ERROR_MSG, IDS_FMTOPEN_FAILED, szFile);
      return(false);
   };
   
   //Enumerate the files in the registry
   CObArray FileList;
   CFileData* pData;
   ReadFileList(FileList);
   DWORD dwSize = FileList.GetSize();
   for (DWORD dwIdx = 0;
        dwIdx < dwSize;
        dwIdx++) 
   {
      //Merge in the file 
      pData = MAKE_FILEDATA(FileList[dwIdx]);
      MergeFile(pData->GetFileName(), File);               
   };

   //Clean up
   File.Flush();
   File.Close();
   {  //Solves scope problem
      for (DWORD dwIdx = 0;
           dwIdx < dwSize;
           dwIdx++)
      {
         pData = MAKE_FILEDATA(FileList[dwIdx]);
         delete pData;
      };
   }; 
   FileList.RemoveAll();

   //It appears that sometimes usertypes.dat can be overwritten,
   //attempt to do so
   CString szSrc = BuildPath(szMSDevDir, IDS_TEMPFILE);
   CString szDest = BuildPath(szMSDevDir, IDS_DATFILE);

   //Delete the old file, if it exists
   if ((GetFileAttributes(szDest) != -1) && !DeleteFile(szDest))
   {  
      //Will have to replace it on shutdown
      if (bGUI)
         DisplayMsg(INFO_MSG, IDS_MERGEDONE);
   }
   else
   {
      //Create the new file
      if (!MoveFile(szSrc, szDest))         
      {
         if (bGUI)
         {
            DisplayMsg(ERROR_MSG, IDS_MOVEFAILED);         
            return(false);
         };
      } else if (bGUI)
      {
            DisplayMsg(INFO_MSG, IDS_RESTART);
      };
   };
   return(true);
};