/******************************************************
 *             		 DSAddIn.cpp
 *
 * Project : MergeKW                       Ver : 1.0
 * Programmer : Michael Taylor	     Date : 8/16/98
 * Language : VC 5.0		            Target : Win32
 * Purpose : This file provides the boiler plate code
 *           for working with DevStudio.
 *****************************************************/
//@DOC

#include "stdafx.h"
#include "MergeKW.h"
#include "DSAddIn.h"
#include "Commands.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern void HandleInitFile ( void );

// This is called when the user first loads the add-in, and on start-up
//  of each subsequent Developer Studio session
STDMETHODIMP CDSAddIn::OnConnection(IApplication* pApp, VARIANT_BOOL bFirstTime,
		long dwCookie, VARIANT_BOOL* OnConnection)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	// Store info passed to us
	IApplication* pApplication = NULL;
	if (FAILED(pApp->QueryInterface(IID_IApplication, (void**) &pApplication))
		|| pApplication == NULL)
	{
		*OnConnection = VARIANT_FALSE;
		return S_OK;
	}

	m_dwCookie = dwCookie;

	// Create command dispatch, send info back to DevStudio
	CCommandsObj::CreateInstance(&m_pCommands);
	m_pCommands->AddRef();

	// The QueryInterface above AddRef'd the Application object.  It will
	//  be Release'd in CCommand's destructor.
	m_pCommands->SetApplicationObject(pApplication);

	// (see stdafx.h for the definition of VERIFY_OK)

	VERIFY_OK(pApplication->SetAddInInfo((long) AfxGetInstanceHandle(),
		(LPDISPATCH) m_pCommands, IDR_TOOLBAR_MEDIUM, IDR_TOOLBAR_LARGE, m_dwCookie));

	// The command name should not be localized to other languages.  The 
	//  tooltip, command description, and other strings related to this
	//  command are stored in the string table (IDS_CMD_STRING) and should
	//  be localized.

   //Command 1 is the Merge command
	LPCTSTR szCommand = _T("MergeKWCommand");
	VARIANT_BOOL bRet;
	CString strCmdString;
	strCmdString.LoadString(IDS_CMD1_STRING);
	strCmdString = szCommand + strCmdString;
	CComBSTR bszCmdString(strCmdString);
	CComBSTR bszMethod(_T("MergeKWCommandMethod"));
	CComBSTR bszCmdName(szCommand);
	VERIFY_OK(pApplication->AddCommand(bszCmdString, bszMethod, 0, m_dwCookie, &bRet));
	if (bRet == VARIANT_FALSE)
	{
		// AddCommand failed because a command with this name already
		//  exists.  You may try adding your command under a different name.
		//  Or, you can fail to load as we will do here.
		*OnConnection = VARIANT_FALSE;
		return S_OK;
	}

	// Add toolbar buttons only if this is the first time the add-in
	//  is being loaded.  Toolbar buttons are automatically remembered
	//  by Developer Studio from session to session, so we should only
	//  add the toolbar buttons once.
	if (bFirstTime == VARIANT_TRUE)
	{
		VERIFY_OK(pApplication->
			AddCommandBarButton(dsGlyph, bszCmdName, m_dwCookie));

      //Handle a possible initialization file
      HandleInitFile();
	}

   //Command 2 is the Add command
   LPCTSTR szCommand2 = TEXT("AddKWCommand");
   strCmdString.LoadString(IDS_CMD2_STRING);
   strCmdString = szCommand2 + strCmdString;
   bszCmdString = strCmdString;
   bszMethod = TEXT("AddKWCommandMethod");
   bszCmdName = szCommand2;
   VERIFY_OK(pApplication->AddCommand(bszCmdString, bszMethod, 1, m_dwCookie, &bRet));
   if (bRet == VARIANT_FALSE)
   {
      *OnConnection = VARIANT_FALSE;
      return S_OK;
   };

	// Add toolbar buttons only if this is the first time the add-in
	//  is being loaded.  Toolbar buttons are automatically remembered
	//  by Developer Studio from session to session, so we should only
	//  add the toolbar buttons once.
	if (bFirstTime == VARIANT_TRUE)
	{
		VERIFY_OK(pApplication->
			AddCommandBarButton(dsGlyph, bszCmdName, m_dwCookie));
	}

	*OnConnection = VARIANT_TRUE;
	return S_OK;
}

// This is called on shut-down, and also when the user unloads the add-in
STDMETHODIMP CDSAddIn::OnDisconnection(VARIANT_BOOL bLastTime)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

   CFileFind Find;	

	m_pCommands->Release();
	m_pCommands = NULL;

   //Here is where we will replace the current DAT file with
   //the new one.  The DAT file has already been released by
   //MSDEV, so we can replace it.
   //Build the temporary DAT filename
   CString szTempFile = BuildPath(GetMSDevDir(), IDS_TEMPFILE);
   
   //Build the actual DAT filename
   CString szDatFile = BuildPath(GetMSDevDir(), IDS_DATFILE);

   //Does the temporary file exist
   if (Find.FindFile(szTempFile, 0))
   {
      //Delete the old USERTYPE.DAT
      DeleteFile(szDatFile);

      //Needed for bug with CString and variable arguments
      TCHAR szBuffer[MAX_PATH];

      //And rename the new one
      if (!MoveFile(szTempFile, szDatFile))
      {
         TCHAR szBuffer2[MAX_PATH];
         lstrcpy(szBuffer, szTempFile);
         lstrcpy(szBuffer2, szDatFile);
         DisplayMsg(ERROR_MSG, IDS_FMTRENAME_FAILED, szBuffer, szBuffer2);
      }
      else
      {
         lstrcpy(szBuffer, szDatFile);
         DisplayMsg(INFO_MSG, IDS_FMTCREATE_SUCCESS, szBuffer);
      };
   };

	return S_OK;
}

